#include "mixerctr.h"

MixerCtr::MixerCtr()
{
	m_hMixer = NULL;
	m_dwVolumeControlID  = 0xffffffff;
	m_dwMuteControlID    = 0xffffffff;
}

MixerCtr::~MixerCtr()
{
	ReleaseMixer();
}

//hw  MM_MIXM_LINE_CHANGE, MM_MIXM_CONTROL_CHANGE 
bool MixerCtr::InitMixer(HWND hw)
{
	if (m_hMixer) return true;
	if (mixerOpen(&m_hMixer,0,(DWORD)hw,0,CALLBACK_WINDOW) != MMSYSERR_NOERROR)
        return false;
	//if(mixerGetDevCaps((UINT)m_hMixer, &m_mxcaps, sizeof(MIXERCAPS))!=MMSYSERR_NOERROR) return false;
	return	true;
}

void MixerCtr::ReleaseMixer(void)
{
	if (m_hMixer) mixerClose(m_hMixer);
	m_hMixer = NULL;
	m_dwVolumeControlID  = 0xffffffff;
	m_dwMuteControlID    = 0xffffffff;
}

//-----------------------------------------------------------------------------
//             @@@@@@@@@@@@{[
//-----------------------------------------------------------------------------

bool MixerCtr::InitMasterVolumeControl(void)
{
	MIXERLINE mxl;
	MIXERCONTROL mxc;
	MIXERLINECONTROLS mxlc;

	if (m_dwVolumeControlID != 0xffffffff) return true;
	if (InitMixer(NULL) == false) return false;

	mxl.cbStruct = sizeof(MIXERLINE);
	mxl.dwComponentType = MIXERLINE_COMPONENTTYPE_DST_SPEAKERS;
	if (mixerGetLineInfo((HMIXEROBJ)m_hMixer,&mxl,
				MIXER_OBJECTF_HMIXER | MIXER_GETLINEINFOF_COMPONENTTYPE)!=MMSYSERR_NOERROR)
        return false;

	mxlc.cbStruct = sizeof(MIXERLINECONTROLS);
	mxlc.dwLineID = mxl.dwLineID;
	mxlc.dwControlType = MIXERCONTROL_CONTROLTYPE_VOLUME;
	mxlc.cControls = 1;
	mxlc.cbmxctrl = sizeof(MIXERCONTROL);
	mxlc.pamxctrl = &mxc;
	if (mixerGetLineControls((HMIXEROBJ)m_hMixer,&mxlc,
			   MIXER_OBJECTF_HMIXER | MIXER_GETLINECONTROLSF_ONEBYTYPE) != MMSYSERR_NOERROR)
        return false;

	m_dwMinimum = mxc.Bounds.dwMinimum;
	m_dwMaximum = mxc.Bounds.dwMaximum;
	m_dwVolumeControlID = mxc.dwControlID;

	return true;
}

bool MixerCtr::GetMasterVolume(int *Val)
{
	MIXERCONTROLDETAILS_UNSIGNED mxcdVolume;
	MIXERCONTROLDETAILS mxcd;

	if (InitMasterVolumeControl() == false) return false;

	mxcd.cbStruct = sizeof(MIXERCONTROLDETAILS);
	mxcd.dwControlID = m_dwVolumeControlID;
	mxcd.cChannels = 1;
	mxcd.cMultipleItems = 0;
	mxcd.cbDetails = sizeof(MIXERCONTROLDETAILS_UNSIGNED);
	mxcd.paDetails = &mxcdVolume;
	if (mixerGetControlDetails((HMIXEROBJ)m_hMixer,&mxcd,
				 MIXER_OBJECTF_HMIXER | MIXER_GETCONTROLDETAILSF_VALUE)!=MMSYSERR_NOERROR)
        return false;

	*Val = ((mxcdVolume.dwValue-m_dwMinimum)*100 + (m_dwMaximum-m_dwMinimum)/2)/(m_dwMaximum-m_dwMinimum);

	return true;
}

bool MixerCtr::SetMasterVolume(int Val)
{
	MIXERCONTROLDETAILS_UNSIGNED	mxcdVolume;
	MIXERCONTROLDETAILS				mxcd;

	if (InitMasterVolumeControl() == false) return false;

	mxcdVolume.dwValue=Val*(m_dwMaximum-m_dwMinimum)/100+m_dwMinimum;

	mxcd.cbStruct = sizeof(MIXERCONTROLDETAILS);
	mxcd.dwControlID = m_dwVolumeControlID;
	mxcd.cChannels = 1;
	mxcd.cMultipleItems = 0;
	mxcd.cbDetails = sizeof(MIXERCONTROLDETAILS_UNSIGNED);
	mxcd.paDetails = &mxcdVolume;
	if (mixerSetControlDetails((HMIXEROBJ)m_hMixer,&mxcd,
				 MIXER_OBJECTF_HMIXER | MIXER_SETCONTROLDETAILSF_VALUE)!=MMSYSERR_NOERROR)
		return false;
	return true;
}

bool MixerCtr::UpDownMasterVolume(int dif)
{
	int	Val;

	if (GetMasterVolume(&Val) == false) return false;

	Val += dif;
	if (Val>100) {
        Val=100;
	} else if(Val<0) {
        Val=0;
    }

	if (SetMasterVolume(Val) == false) return false;

	return true;
}

//-----------------------------------------------------------------------------
//             @@@@@@@@@@@@~[g
//-----------------------------------------------------------------------------

bool MixerCtr::InitMasterMuteControl()
{
	MIXERLINE mxl;
	MIXERCONTROL mxc;
	MIXERLINECONTROLS mxlc;

	if (m_dwMuteControlID != 0xffffffff) return true;
	if (InitMixer(NULL) == false) return false;

	mxl.cbStruct = sizeof(MIXERLINE);
	mxl.dwComponentType = MIXERLINE_COMPONENTTYPE_DST_SPEAKERS;
	if (mixerGetLineInfo((HMIXEROBJ)m_hMixer,&mxl,
				MIXER_OBJECTF_HMIXER | MIXER_GETLINEINFOF_COMPONENTTYPE)!=MMSYSERR_NOERROR)
        return false;

	mxlc.cbStruct = sizeof(MIXERLINECONTROLS);
	mxlc.dwLineID = mxl.dwLineID;
	mxlc.dwControlType = MIXERCONTROL_CONTROLTYPE_MUTE;
	mxlc.cControls = 1;
	mxlc.cbmxctrl = sizeof(MIXERCONTROL);
	mxlc.pamxctrl = &mxc;
	if (mixerGetLineControls((HMIXEROBJ)m_hMixer,&mxlc,
			   MIXER_OBJECTF_HMIXER | MIXER_GETLINECONTROLSF_ONEBYTYPE) != MMSYSERR_NOERROR)
        return false;

	m_dwMuteControlID = mxc.dwControlID;

	return true;
}

bool MixerCtr::GetMasterMute(bool *Val)
{
	MIXERCONTROLDETAILS_BOOLEAN mxcdMute;
	MIXERCONTROLDETAILS mxcd;

	if (InitMasterMuteControl() == false) return false;

	mxcd.cbStruct = sizeof(MIXERCONTROLDETAILS);
	mxcd.dwControlID = m_dwMuteControlID;
	mxcd.cChannels = 1;
	mxcd.cMultipleItems = 0;
	mxcd.cbDetails = sizeof(MIXERCONTROLDETAILS_BOOLEAN);
	mxcd.paDetails = &mxcdMute;
	if (mixerGetControlDetails((HMIXEROBJ)m_hMixer,&mxcd,
				 MIXER_OBJECTF_HMIXER | MIXER_GETCONTROLDETAILSF_VALUE)!=MMSYSERR_NOERROR)
        return false;

	*Val = mxcdMute.fValue;

	return true;
}

bool MixerCtr::SetMasterMute(bool Val)
{
	MIXERCONTROLDETAILS_BOOLEAN mxcdMute;
	MIXERCONTROLDETAILS mxcd;

	if (InitMasterMuteControl() == false) return false;

	mxcdMute.fValue=Val;

	mxcd.cbStruct = sizeof(MIXERCONTROLDETAILS);
	mxcd.dwControlID = m_dwMuteControlID;
	mxcd.cChannels = 1;
	mxcd.cMultipleItems = 0;
	mxcd.cbDetails = sizeof(MIXERCONTROLDETAILS_BOOLEAN);
	mxcd.paDetails = &mxcdMute;
	if (mixerSetControlDetails((HMIXEROBJ)m_hMixer,&mxcd,
				 MIXER_OBJECTF_HMIXER | MIXER_SETCONTROLDETAILSF_VALUE)!=MMSYSERR_NOERROR)
        return false;
	
	return true;
}

bool MixerCtr::ReverseMasterMute(void)
{
	bool bMute;
	if (GetMasterMute(&bMute) == false) return false;
	return SetMasterMute(!bMute);
}

